-- ============================================================================
-- == HalonaTools - A collection of tools required by members of the         ==
-- ==               Alliance PVE Raiding Guild "Halona" of the Silvermoon EU ==
-- ==               Realm of Blizzard Entertainment's "World of Warcraft"    ==
-- ==               MMORPG. These tools are provided in the form of an addon ==
-- ==               that supplements those controls found within the default ==
-- ==               UI provided by the game and those provided by the        ==
-- ==               authors of other third-party UI addons.                  ==
-- ==                                                                        ==
-- ==               This addon is based upon the Ace3 library framework and  ==
-- ==               additionally supports other standards such as the LDB    ==
-- ==               (aka: LibDataBroker) method for enabling integration of  ==
-- ==               this addon with a wide-range of LDB-compatible "Display" ==
-- ==               addons.                                                  ==
-- ==                                                                        ==
-- ==               Development of this addon is managed via the resources   ==
-- ==               provided by the WoWAce community as follows.             ==
-- ==                                                                        ==
-- ==               Home-Page: http://www.wowace.com/projects/HalonaTools    ==
-- ==               SVN......: svn://svn.wowace.com/projects/halonatools     ==
-- ==               Download.: http://wow.curse.com/addons/wow/HalonaTools   ==
-- ==               Author...: broliant@broliant.com                         ==
-- ============================================================================

HalonaTools   = LibStub("AceAddon-3.0"):NewAddon("HalonaTools", "AceConsole-3.0", "AceEvent-3.0")
local L       = LibStub("AceLocale-3.0"):GetLocale("HalonaTools")

-- ============================================================================
-- == OnInitialize()                                                         ==
-- ==                                                                        ==
-- == This function is called when the addon is initialised.                 ==
-- ============================================================================

function HalonaTools:OnInitialize()
end

-- ============================================================================
-- == OnEnable()                                                             ==
-- ==                                                                        ==
-- == This function is called when the addon is enabled.                     ==
-- ============================================================================

function HalonaTools:OnEnable()

  HTConsole:BroadcastMessageAVB()           -- Addon Version Broadcast
  HalonaTools:JoinGuildRaidEvent()          -- Attempt to Join Guild Raid Event.

end

-- ============================================================================
-- == OnEnable()                                                             ==
-- ==                                                                        ==
-- == This function is called when the addon is disabled.                    ==
-- ============================================================================

function HalonaTools:OnDisable()
end

-- ============================================================================
-- == DoHeartbeat()                                                          ==
-- ==                                                                        ==
-- == This function is intended to provide a place where non-critical tasks  ==
-- == that need to be performed periodically can be called from. Typically   ==
-- == each HT component may have an instance of this method and each will    ==
-- == be called in turn by the corresponding function in HTConsole which is  ==
-- == considered to be the master-heartbeat component.                       ==
-- ============================================================================

function HalonaTools:DoHeartbeat()

  HalonaTools:GuildAdminChecks()
  HalonaTools:AutoRaidChecks()

end

-- ============================================================================
-- == BeginRaidInvitationPeriod(characterName)                               ==
-- ==                                                                        ==
-- == This function handles the initiation of the raid invitation period.    ==
-- == Specifically, it sends an announcement to the GUILD chat channel and   ==
-- == updates the GMOTD to indicate that the period is in effect and to      ==
-- == advise members who to contact for an invitation to the raid.           ==
-- ==                                                                        ==
-- == Additionally, a broadcast message is sent out via the addon channel so ==
-- == that other players using HalonaTools may detect the event and offer    ==
-- == the player the option of requesting an invitation automatically.       ==
-- ============================================================================

function HalonaTools:BeginRaidInvitationPeriod(characterName)

  local gmotdText = string.gsub(L["ht-raid-invites-begin-gmotd-01"], "CHARACTER_NAME", characterName)
  local gchatText = string.gsub(L["ht-raid-invites-begin-gchat-01"], "CHARACTER_NAME", characterName)

  if HTCommon:IsOfficer(characterName) then
    GuildSetMOTD(gmotdText)
  end

  HTConsole:SendTitleBar("GUILD")
  HTConsole:SendWrappedMessage(L["ht-raid-invites-begin-title-01"], "GUILD", nil, nil)
  HTConsole:SendTitleBar("GUILD")
  HTConsole:SendWrappedMessage(gchatText, "GUILD", nil, nil)
  HTConsole:SendTitleBar("GUILD")

  --
  -- Send the raid invitation announcement addon message.
  --
  local strCharacterName = HTCommon:FindPlayerByCharacterName(characterName)
  local simpleName       = HTCommon:GetSimpleCharacterName(strCharacterName)

  SendAddonMessage(L["con-addonmessage-prefix-htria"], simpleName, "GUILD")

end

-- ============================================================================
-- == BeginRaidSession(characterName)                                        ==
-- ==                                                                        ==
-- == This function handles the initiation of the raid session.              ==
-- == Specifically, it sends an announcement to the GUILD chat channel and   ==
-- == updates the GMOTD to indicate that the raid is in effect and to        ==
-- == advise members who to contact in order to join the raid queue.         ==
-- ==                                                                        ==
-- == Additionally, a broadcast message is sent out via the addon channel so ==
-- == that other players using HalonaTools may detect the event and offer    ==
-- == the player the option of requesting a queue spot automatically.        ==
-- ============================================================================

function HalonaTools:BeginRaidSession(characterName)

  local gmotdText = string.gsub(L["ht-raid-session-begin-gmotd-01"], "CHARACTER_NAME", characterName)
  local gchatText = string.gsub(L["ht-raid-session-begin-gchat-01"], "CHARACTER_NAME", characterName)

  if HTCommon:IsOfficer(characterName) then
    GuildSetMOTD(gmotdText)
  end

  HTConsole:SendTitleBar("GUILD")
  HTConsole:SendWrappedMessage(L["ht-raid-session-begin-title-01"], "GUILD", nil, nil)
  HTConsole:SendTitleBar("GUILD")
  HTConsole:SendWrappedMessage(gchatText, "GUILD", nil, nil)
  HTConsole:SendTitleBar("GUILD")

  --
  -- Send the raid invitation announcement addon message.
  --
  local strCharacterName = HTCommon:FindPlayerByCharacterName(characterName)
  local simpleName       = HTCommon:GetSimpleCharacterName(strCharacterName)

  SendAddonMessage(L["con-addonmessage-prefix-htrqa"], simpleName, "GUILD")

end

-- ============================================================================
-- == EndRaidSession(characterName)                                          ==
-- ==                                                                        ==
-- == This function handles the termination of the raid session.             ==
-- ============================================================================

function HalonaTools:EndRaidSession(characterName)

  local gmotdText = string.gsub(L["ht-raid-session-end-gmotd-01"], "CHARACTER_NAME", characterName)
  local gchatText = string.gsub(L["ht-raid-session-end-gchat-01"], "CHARACTER_NAME", characterName)

  if HTCommon:IsOfficer(characterName) then
    GuildSetMOTD(gmotdText)
  end

  HTConsole:SendTitleBar("GUILD")
  HTConsole:SendWrappedMessage(L["ht-raid-session-end-title-01"], "GUILD", nil, nil)
  HTConsole:SendTitleBar("GUILD")
  HTConsole:SendWrappedMessage(gchatText, "GUILD", nil, nil)
  HTConsole:SendTitleBar("GUILD")

  --
  -- Send the raid end announcement addon message.
  --
  local strCharacterName = HTCommon:FindPlayerByCharacterName(characterName)
  local simpleName       = HTCommon:GetSimpleCharacterName(strCharacterName)

  SendAddonMessage(L["con-addonmessage-prefix-htrea"], simpleName, "GUILD")

end

-- ============================================================================
-- == EndRaidSessionQuietly(characterName)                                   ==
-- ==                                                                        ==
-- == This function handles the termination of the raid session, without     ==
-- == making a chat-based announcement.                                      ==
-- ============================================================================

function HalonaTools:EndRaidSessionQuietly(characterName)

  local gmotdText = string.gsub(L["ht-raid-session-end-gmotd-01"], "CHARACTER_NAME", characterName)

  if HTCommon:IsOfficer(characterName) then
    GuildSetMOTD(gmotdText)
  end

  --
  -- Send the raid end announcement addon message.
  --
  local strCharacterName = HTCommon:FindPlayerByCharacterName(characterName)
  local simpleName       = HTCommon:GetSimpleCharacterName(strCharacterName)

  SendAddonMessage(L["con-addonmessage-prefix-htrea"], simpleName, "GUILD")

end

-- ============================================================================
-- == JoinGuildRaidEvent()                                                   ==
-- ==                                                                        ==
-- == This function examines the GMOTD to determine whether a guild raid     ==
-- == event is either about to take place or currently taking place and      ==
-- == places a request on the user's behalf accordingly if there is.         ==
-- ============================================================================

function HalonaTools:JoinGuildRaidEvent()

  if HTCommon:IsPlayerInParty() or HTCommon:IsPlayerInRaid() then
    HTConsole:SendChatMessage(L["ht-raid-check-skipped"] , "CHAT", nil, nil)
    return
  end

  -- == -----------------------------------------------------------------------
  -- == -- Retrieve the GMOTD, decode into component parts. (ie: Words.)     --
  -- == -----------------------------------------------------------------------
  local gmotdActualText     = HTData:SafeString(GetGuildRosterMOTD())
  local gmotdActualWordList = HTData:StringSplit(gmotdActualText, " ")
  local gmotdRIAWordList    = HTData:StringSplit(L["ht-raid-invites-begin-gmotd-01"], " ")
  local gmotdRQAWordList    = HTData:StringSplit(L["ht-raid-session-begin-gmotd-01"], " ")

  local riaMatchText = gmotdRIAWordList[1] .. " " .. gmotdRIAWordList[2] .. " " .. gmotdRIAWordList[3] .. " " .. gmotdRIAWordList[4] .. " " .. gmotdRIAWordList[5] .. " " .. gmotdRIAWordList[6]
  local riaMatchLen  = string.len(riaMatchText)

  local rqaMatchText = gmotdRQAWordList[1] .. " " .. gmotdRQAWordList[2] .. " " .. gmotdRQAWordList[3] .. " " .. gmotdRQAWordList[4] .. " " .. gmotdRQAWordList[5]
  local rqaMatchLen  = string.len(rqaMatchText)

  -- == -----------------------------------------------------------------------
  -- == -- Test the GMOTD: Is the raid invitation period open?               --
  -- == -----------------------------------------------------------------------
  if string.len(gmotdActualText) > riaMatchLen then
    if string.sub(gmotdActualText, 1, riaMatchLen) == riaMatchText then
      if not HTData:SafeString(gmotdActualWordList[12]) == UnitName("player") then
        HTGUI:PopupAskRequestRaidInvitation(HTData:SafeString(gmotdActualWordList[12]), UnitName("player"))
        return
      end
    end
  end

  -- == -----------------------------------------------------------------------
  -- == -- Test the GMOTD: Is a raid currently in progress?                  --
  -- == -----------------------------------------------------------------------
  if string.len(gmotdActualText) > rqaMatchLen then
    if string.sub(gmotdActualText, 1, rqaMatchLen) == rqaMatchText then
      if not HTData:SafeString(gmotdActualWordList[12]) == UnitName("player") then
        HTGUI:PopupAskRequestRaidQueueListing(HTData:SafeString(gmotdActualWordList[12]), UnitName("player"))
        return
      end
    end
  end

  -- No raid event in progress, report then return.
  HTConsole:SendChatMessage(L["ht-raid-not-in-progress-01"], "CHAT", nil, nil)

end

-- ============================================================================
-- == CreateDBMTimer(duration, name, broadcastToRaid)                        ==
-- ==                                                                        ==
-- == This function creates a new DBM (Deadly Boss-Mods) timer where the     ==
-- == duration, name and whether the timer is to be broadcast to the raid    ==
-- == are provided as parameters.                                            ==
-- ============================================================================

function HalonaTools:CreateDBMTimer(duration, name, broadcastToRaid)

  DBM:CreatePizzaTimer(duration, name, broadcastToRaid)

end

-- ============================================================================
-- == ShowVersions()                                                         ==
-- ==                                                                        ==
-- == This function simply displays the versions of HT currently being used  ==
-- == by those currently online.                                             ==
-- ============================================================================

function HalonaTools:ShowVersions()

  local currentVersions = HTConsole:GetVersions() -- HTConfig:GetVersions(nil)
  local myVersionName   = L["addon-version-name"]
  local myVersionNumber = L["addon-version-number"]

  HTConsole:SendChatMessage("[HT] - Version Check - Guild Members.", "CHAT", nil, nil)

  for i,v in ipairs(currentVersions) do

    local args = HTData:StringSplit(v, "::")

    local currentUsername      = args[1]
    local currentVersionName   = args[2]
    local currentVersionNumber = args[3]

    local messageText = "|c0ffffff00" .. currentUsername .. "|r"

    messageText = messageText .. " - |cff00c0ff" .. currentVersionName .. "|r:"

    if currentVersionNumber < myVersionNumber then
      messageText = messageText .. "|c0ffff0000" .. currentVersionNumber .. "|r"
    elseif currentVersionNumber > myVersionNumber then
      messageText = messageText .. "|c0ff00ff00" .. currentVersionNumber .. "|r"
    else
      messageText = messageText .. "|c0ffffff00" .. currentVersionNumber .. "|r"
    end

    HTConsole:SendChatMessage(messageText, "CHAT", nil, nil)

  end
end

-- ============================================================================
-- == ShowReportUsageStatus(channel)                                         ==
-- ==                                                                        ==
-- == This function displays a report of HT's usage status within the        ==
-- == members of the guild currently online.                                 ==
-- ============================================================================

function HalonaTools:ShowReportUsageStatus(channel)

  -- Fetch all data required for report.
  local userDataTypeCurrent = HTConsole:GetHTUserData(L["con-ht-userdata-type-current"])
  local userDataTypeOld     = HTConsole:GetHTUserData(L["con-ht-userdata-type-old"])
  local userDataTypeAny     = HTConsole:GetHTUserData(L["con-ht-userdata-type-any"])
  local userDataTypeNone    = HTConsole:GetHTUserData(L["con-ht-userdata-type-none"])
  local maxVersionName      = HTConsole:GetMaxVersionName()
  local maxVersionNumber    = HTConsole:GetMaxVersionNumber()
  local goodVersionText     = HTData:BuildStringFromTable(userDataTypeCurrent, L["ht-usage-status-uptodate"],  ", ")
  local oldVersionText      = HTData:BuildStringFromTable(userDataTypeOld,     L["ht-usage-status-outofdate"], ", ")
  local noVersionText       = HTData:BuildStringFromTable(userDataTypeNone,    L["ht-usage-status-missing"],   ", ")
  local maxVersionText      = L["ht-usage-status-maxversion"]
  maxVersionText            = string.gsub(maxVersionText, "VERSION_NAME", maxVersionName)
  maxVersionText            = string.gsub(maxVersionText, "VERSION_NUMBER", maxVersionNumber)

  -- Report Usage Status to specified channel.
  HTConsole:SendTitleBar(channel)
  HTConsole:SendWrappedMessage(L["ht-usage-status-report-title"], channel, nil, nil)
  HTConsole:SendTitleBar(channel)
  HTConsole:SendWrappedMessage(maxVersionText, channel, nil, nil)

  -- Only output each of the following if there are users contained in the table.
  if #userDataTypeCurrent > 0 then HTConsole:SendWrappedMessage(goodVersionText, channel, nil, nil) end
  if #userDataTypeOld     > 0 then HTConsole:SendWrappedMessage(oldVersionText,  channel, nil, nil) end
  if #userDataTypeNone    > 0 then HTConsole:SendWrappedMessage(noVersionText,   channel, nil, nil) end

  HTConsole:SendTitleBar(channel)

end

-- ============================================================================
-- == AutoRaidChecks()                                                       ==
-- ==                                                                        ==
-- == This function is responsible for running the auto raid checks that     ==
-- == attempt to manage various raid-related features/functions.             ==
-- ============================================================================

function HalonaTools:AutoRaidChecks()

  HalonaTools:RaidCheckNoMasterLooterSet()
  HalonaTools:RaidCheckAutoSetMasterLoot()
  HalonaTools:RaidCheckCombatDetectedWhenPlayModeIsInvites()

end

-- ============================================================================
-- == RaidCheckNoMasterLooterSet()                                           ==
-- ==                                                                        ==
-- == This function displays a warning when the user is in a raid and no     ==
-- == master looter has been set, if certain conditions are met. Those       ==
-- == conditions are:                                                        ==
-- ==                                                                        ==
-- == -> The user is a guild officer or guild master.                        ==
-- == -> The user is a raid assistant or raid leader.                        ==
-- ==                                                                        ==
-- ============================================================================

function HalonaTools:RaidCheckNoMasterLooterSet()

  if HTCommon:IsPlayerInRaid() then
    local characterName = UnitName("player")
    if HTCommon:IsOfficer(characterName) then
      if HTCommon:IsCharacterRaidLeader(characterName) or HTCommon:IsCharacterRaidAssistant(characterName) then
        if HTCommon:GetCurrentGuildPlayMode() == L["cmn-guild-playmode-invites"] or HTCommon:GetCurrentGuildPlayMode() == L["cmn-guild-playmode-raiding"] then
          local masterLooter = HTCommon:GetMasterLooterCharacterName()
          if masterLooter == L["all-data-unset"] then
            HTConsole:SendChatMessage(L["ht-arc-no-master-looter-set"], "OFFICER", nil, nil)
          end
        end
      end
    end
  end

end

-- ============================================================================
-- == RaidCheckAutoSetMasterLoot()                                           ==
-- ==                                                                        ==
-- == This function attempts to set the loot method to Master Loot under     ==
-- == various conditions as follows:                                         ==
-- ==                                                                        ==
-- == Condition 1: Player is in raid and is RL.                              ==
-- ==              Auto-Set Master Loot feature enabled.                     ==
-- ==              No ML is currently set.                                   ==
-- ==              Auto-Select Master Looter feature is enabled.             ==
-- ==              ML is set to best candidate from all current raid members.==
-- ==                                                                        ==
-- == Condition 2  Player is in raid and is RL.                              ==
-- ==              Auto-Set Master Loot feature enabled.                     ==
-- ==              No ML is currently set.                                   ==
-- ==              Auto-Select Master Looter feature is disabled.            ==
-- ==              ML is set to player as the RL / default-ML candidate.     ==
-- ==                                                                        ==
-- == Condition 3  Player is in raid and is RL.                              ==
-- ==              Auto-Set Master Loot feature enabled.                     ==
-- ==              An ML is currently set.                                   ==
-- ==              Auto-Select Master Looter feature is enabled.             ==
-- ==           ** Guild Play Mode is currently "Invites"                    ==
-- ==              ML is set to best candidate from all current raid members.==
-- ==                                                                        ==
-- == ** We allow an existing ML to be replaced automatically by a better    ==
-- ==    candidate during the "Invites" period but once the proper raid      ==
-- ==    session is in effect, we no longer allow the ML to be auto-changed. ==
-- ============================================================================

function HalonaTools:RaidCheckAutoSetMasterLoot()

  -- Return immediately if the user is NOT in a raid OR is NOT the raid leader.
  if HTCommon:IsPlayerInRaid() == false or HTCommon:IsCharacterRaidLeader(UnitName("player")) == false then return end

  -- Only process the following IF the feature is enabled.
  if HTConfig:GetIsEnabledAutoSetMasterLoot(nil) == true then

    -- If no ML is set, we need to set one.
    if HTCommon:GetMasterLooterCharacterName() == L["all-data-unset"] then

      local masterLooterName = L["all-data-unset"]

      if HTConfig:GetIsEnabledAutoSelectMasterLooter(nil) == true then
        masterLooterName = HTCommon:GetBestMasterLooterCandidateName()
      end

      if masterLooterName == L["all-data-unset"] then
        masterLooterName = UnitName("player") -- The player is the RL remember.
      end

      SetLootMethod(L["ht-arc-auto-set-ml-method"], masterLooterName, L["ht-arc-auto-set-ml-threshold"])

      HTConsole:SendChatMessage(string.gsub(L["ht-arc-auto-set-ml"], "CHARACTER_NAME", masterLooterName), "OFFICER", nil, nil)

    -- An ML is set but we MIGHT need to see if we can set a better one.
    else

      -- We are permitted to set a new ML during the invites period IF the option is enabled.
      if HTCommon:GetCurrentGuildPlayMode() == L["cmn-guild-playmode-invites"] and HTConfig:GetIsEnabledAutoSelectMasterLooter(nil) == true then
        local masterLooterName = HTCommon:GetBestMasterLooterCandidateName()

        if masterLooterName ~= L["all-data-unset"] then
          SetLootMethod(L["ht-arc-auto-set-ml-method"], masterLooterName, L["ht-arc-auto-set-ml-threshold"])

          HTConsole:SendChatMessage(string.gsub(L["ht-arc-auto-set-ml"], "CHARACTER_NAME", masterLooterName), "OFFICER", nil, nil)
        end
      end
    end

  end

end

-- ============================================================================
-- == RaidCheckCombatDetectedWhenPlayModeIsInvites()                         ==
-- ==                                                                        ==
-- == This function displays a warning when the user is in a raid and combat ==
-- == is detected whilst the guild play mode is still set to "invites"       ==
-- ==                                                                        ==
-- == -> The user is a guild officer or guild master.                        ==
-- == -> The user is a raid assistant or raid leader.                        ==
-- ==                                                                        ==
-- ============================================================================

function HalonaTools:RaidCheckCombatDetectedWhenPlayModeIsInvites()

  if HTCommon:IsPlayerInRaid() then
    local characterName = UnitName("player")
    if HTCommon:IsOfficer(characterName) then
      if HTCommon:IsCharacterRaidLeader(characterName) or HTCommon:IsCharacterRaidAssistant(characterName) then
        if HTCommon:GetCurrentGuildPlayMode() == L["cmn-guild-playmode-invites"] and HTConfig:GetIsPlayerInCombat(nil) == true then
          HTConsole:SendChatMessage(L["ht-arc-combat-when-play-mode-invites"], "OFFICER", nil, nil)
        end
      end
    end
  end

end

-- ============================================================================
-- == GuildAdminChecks()                                                     ==
-- ==                                                                        ==
-- == This function is responsible for running the guild admin checks that   ==
-- == correspond to the player's rank.                                       ==
-- ============================================================================

function HalonaTools:GuildAdminChecks()

  HalonaTools:GuildAdminOfficerChecks(true)

end

-- ============================================================================
-- == GuildAdminOfficerChecks(doQuietly)                                     ==
-- ==                                                                        ==
-- == This function is responsible for running the guild admin checks that   ==
-- == correspond to officer ranked players.                                  ==
-- ============================================================================

function HalonaTools:GuildAdminOfficerChecks(doQuietly)

  -- HTConsole:Debug(UnitName("player").." officer status: ["..HTData:BooleanAsText(HTCommon:IsOfficer(UnitName("player"))).."]")

  -- Only proceed past this point if the player is an officer of the guild.
  if not HTCommon:IsOfficer(UnitName("player")) then return end

  -- HTConsole:Debug("Performing HalonaTools:GuildAdminOfficerChecks()...")

  -- Check trial member officer notes for conformity. ("Trial yyyy.mm.dd")
  HalonaTools:GuildAdminCheckTrialMemberOfficerNotes(doQuietly)

  -- Check alt character public notes for conformity. ("{main-character-name}")
  HalonaTools:GuildAdminCheckAltCharacterPublicNotes(doQuietly)

  -- Check members have had trial start date removed from officer note field.
  HalonaTools:GuildAdminCheckMemberOfficerNotes(doQuietly)

end

-- ============================================================================
-- == CheckTrialCharacterOfficerNotes(doQuietly)                             ==
-- ==                                                                        ==
-- == This function checks that all trial ranked characters have their       ==
-- == officer note field set correctly. The proper format for the officer    ==
-- == note of a trial ranked member is as follows:                           ==
-- ==                                                                        ==
-- == Trial dd.mm.yyyy                                                       ==
-- ==                                                                        ==
-- ============================================================================

function HalonaTools:GuildAdminCheckTrialMemberOfficerNotes(doQuietly)

  local totalNumberOfGuildMembers = GetNumGuildMembers(true)
  local problemMemberList         = {}

  for index=1, totalNumberOfGuildMembers do
    local name, rank, rankIndex, level, class, zone, note, officernote, online, status = GetGuildRosterInfo(index)
    if HTCommon:IsMainRankedCharacter(name) == true then
      if HTCommon:IsTrialRankedCharacter(name) == true then
        if HalonaTools:IsOfficerNoteValidForTrialMember(officernote) == false then
          table.insert(problemMemberList, name.."=["..officernote.."]")
        end
      end
    end
  end

  table.sort(problemMemberList)

  if #problemMemberList > 0 then
    HTConsole:SendChatMessage(L["ht-admin-trial-officer-notes-title"], "OFFICER", nil, nil)
    HTConsole:SendChatMessage(L["ht-admin-trial-officer-notes-test"], "OFFICER", nil, nil)
    HTConsole:SendChatMessage(L["ht-admin-trial-officer-notes-expected"], "OFFICER", nil, nil)
    HTConsole:SendWrappedMessage(HTData:BuildStringFromTable(problemMemberList, L["ht-admin-trial-officer-notes-failures"], ", "), "OFFICER", nil, nil)
    HTConsole:SendChatMessage(L["ht-admin-trial-officer-notes-alert"], "ERRORFRAME", nil, nil)
  else
    if doQuietly == false then
      HTConsole:SendChatMessage(L["ht-admin-trial-officer-notes-title"], "OFFICER", nil, nil)
      HTConsole:SendChatMessage(L["ht-admin-check-complete"], "OFFICER", nil, nil)
    end
  end

end

-- ============================================================================
-- == IsOfficerNoteValidForTrialMember(officerNote)                          ==
-- ==                                                                        ==
-- == Assumption: New guild members are given the rank of "Trial" and their  ==
-- == officer note is set in the following manner:                           ==
-- ==                                                                        ==
-- == Trial yyyy.mm.dd                                                       ==
-- == 1234567890123456                                                       ==
-- ==                                                                        ==
-- == This function checks the supplied officer-note value and returns true  ==
-- == if it matches the expected format or false otherwise.                  ==
-- ============================================================================

function HalonaTools:IsOfficerNoteValidForTrialMember(officerNote)

   -- Ensure officerNote has a value.
   if officerNote == nil or officerNote == "" then return false end

   -- Ensure officerNote is exactly the right length.
   if string.len(officerNote) ~= 16 then return false end

   -- Ensure beginning matches "Trial "
   if string.sub(officerNote, 1, 6) ~= "Trial " then return false end

   -- Ensure position 11 is a period.
   if string.sub(officerNote, 11, 11) ~= "." then return false end

   -- Ensure position 14 is a period.
   if string.sub(officerNote, 14, 14) ~= "." then return false end

   -- Ensure positions 7-10 are numerals.
   if not tonumber(string.sub(officerNote, 7,10)) then return false end

   -- Ensure positions 12-13 are numerals.
   if not tonumber(string.sub(officerNote, 12,13)) then return false end

   -- Ensure positions 15-16 are numerals.
   if not tonumber(string.sub(officerNote, 15,16)) then return false end

   dateNumber  = tonumber(string.sub(officerNote, 15,16))
   monthNumber = tonumber(string.sub(officerNote, 12,13))
   yearNumber  = tonumber(string.sub(officerNote,  7,10))

   -- Ensure dateNumber is in valid date-range.
   if dateNumber < 0 or dateNumber > 31 then return false end

   -- Ensure monthNumber is in valid date-range.
   if monthNumber < 0 or monthNumber > 12 then return false end

   -- Ensure yearNumber is in valid date-range.
   if yearNumber < 0 or yearNumber > 2000000 then return false end

   -- All tests passed, officerNote is valid!
   return true

end

-- ============================================================================
-- == GuildAdminCheckAltCharacterPublicNotes(doQuietly)                      ==
-- ==                                                                        ==
-- == This function checks that all alt ranked characters have a public note ==
-- == that contains a single word, that word being the name of a main-ranked ==
-- == character within the guild.                                            ==
-- ============================================================================

function HalonaTools:GuildAdminCheckAltCharacterPublicNotes(doQuietly)

  local totalNumberOfGuildMembers = GetNumGuildMembers(true)
  local problemMemberList         = {}

  for index=1, totalNumberOfGuildMembers do
    local name, rank, rankIndex, level, class, zone, note, officernote, online, status = GetGuildRosterInfo(index)
    if HTCommon:IsAltRankedCharacter(name) then
      if HTCommon:IsCharacterInGuild(note) then
        if HTCommon:IsMainRankedCharacter(note) == false then
          table.insert(problemMemberList, name.."=["..note.."]")
        end
      else
        table.insert(problemMemberList, name.."=["..note.."]")
      end
    end
  end

  table.sort(problemMemberList)

  if #problemMemberList > 0 then
    HTConsole:SendChatMessage(L["ht-admin-alt-public-notes-title"], "OFFICER", nil, nil)
    HTConsole:SendChatMessage(L["ht-admin-alt-public-notes-test"], "OFFICER", nil, nil)
    HTConsole:SendChatMessage(L["ht-admin-alt-public-notes-expected"], "OFFICER", nil, nil)
    HTConsole:SendWrappedMessage(HTData:BuildStringFromTable(problemMemberList, L["ht-admin-alt-public-notes-failures"], ", "), "OFFICER", nil, nil)
    HTConsole:SendChatMessage(L["ht-admin-alt-public-notes-alert"], "ERRORFRAME", nil, nil)
  else
    if doQuietly == false then
      HTConsole:SendChatMessage(L["ht-admin-alt-public-notes-title"], "OFFICER", nil, nil)
      HTConsole:SendChatMessage(L["ht-admin-check-complete"], "OFFICER", nil, nil)
    end
  end

end

-- ============================================================================
-- == GuildAdminCheckMemberOfficerNotes(doQuietly)                           ==
-- ==                                                                        ==
-- == This function checks that a promoted member has had the trial note     ==
-- == removed from their officer note when they were promoted.               ==
-- ============================================================================

function HalonaTools:GuildAdminCheckMemberOfficerNotes(doQuietly)

  local totalNumberOfGuildMembers = GetNumGuildMembers(true)
  local problemMemberList         = {}

  for index=1, totalNumberOfGuildMembers do
    local name, rank, rankIndex, level, class, zone, note, officernote, online, status = GetGuildRosterInfo(index)
    if HTCommon:IsMainRankedCharacter(name) == true and HTCommon:IsTrialRankedCharacter(name) == false then
      if HalonaTools:IsOfficerNoteValidForTrialMember(officernote) == true then
        table.insert(problemMemberList, name.."=["..note.."]")
      end
    end
  end

  table.sort(problemMemberList)

  if #problemMemberList > 0 then
    HTConsole:SendChatMessage(L["ht-admin-member-officer-notes-title"], "OFFICER", nil, nil)
    HTConsole:SendChatMessage(L["ht-admin-member-officer-notes-test"], "OFFICER", nil, nil)
    HTConsole:SendChatMessage(L["ht-admin-member-officer-notes-expected"], "OFFICER", nil, nil)
    HTConsole:SendWrappedMessage(HTData:BuildStringFromTable(problemMemberList, L["ht-admin-member-officer-notes-failures"], ", "), "OFFICER", nil, nil)
    HTConsole:SendChatMessage(L["ht-admin-member-officer-notes-alert"], "ERRORFRAME", nil, nil)
  else
    if doQuietly == false then
      HTConsole:SendChatMessage(L["ht-admin-member-officer-notes-title"], "OFFICER", nil, nil)
      HTConsole:SendChatMessage(L["ht-admin-check-complete"], "OFFICER", nil, nil)
    end
  end

end

-- ============================================================================
-- == ListAllOfficers()                                                      ==
-- ==                                                                        ==
-- == This function lists all officer-ranked characters within the guild.    ==
-- ============================================================================

function HalonaTools:ListAllOfficers()

  local totalNumberOfGuildMembers = GetNumGuildMembers(true)
  local characterList             = {}

  for index=1, totalNumberOfGuildMembers do
    local name, rank, rankIndex, level, class, zone, note, officernote, online, status = GetGuildRosterInfo(index)
    if HTCommon:IsMainRankedCharacter(name) == true and HTCommon:IsOfficer(name) == true then
      table.insert(characterList, name)
    end
  end

  table.sort(characterList)

  if #characterList > 0 then
    HTConsole:SendChatMessage(L["ht-list-all-officer-mains-title"], "OFFICER", nil, nil)
    HTConsole:SendWrappedMessage(HTData:BuildStringFromTable(characterList, "", ", "), "OFFICER", nil, nil)
  end

end

-- ============================================================================
-- == ListAllMembers()                                                       ==
-- ==                                                                        ==
-- == This function lists all member-ranked characters within the guild.     ==
-- ============================================================================

function HalonaTools:ListAllMembers()

  local totalNumberOfGuildMembers = GetNumGuildMembers(true)
  local characterList             = {}

  for index=1, totalNumberOfGuildMembers do
    local name, rank, rankIndex, level, class, zone, note, officernote, online, status = GetGuildRosterInfo(index)
    if HTCommon:IsMemberRankedCharacter(name) == true then
      table.insert(characterList, name)
    end
  end

  table.sort(characterList)

  if #characterList > 0 then
    HTConsole:SendChatMessage(L["ht-list-all-member-mains-title"], "OFFICER", nil, nil)
    HTConsole:SendWrappedMessage(HTData:BuildStringFromTable(characterList, "", ", "), "OFFICER", nil, nil)
  end

end

-- ============================================================================
-- == ListAllActiveMembers()                                                 ==
-- ==                                                                        ==
-- == This function lists all active member-ranked characters within the     ==
-- == guild.                                                                 ==
-- ============================================================================

function HalonaTools:ListAllActiveMembers()

  local totalNumberOfGuildMembers = GetNumGuildMembers(true)
  local characterList             = {}

  for index=1, totalNumberOfGuildMembers do
    local name, rank, rankIndex, level, class, zone, note, officernote, online, status = GetGuildRosterInfo(index)
    if HTCommon:IsMemberRankedCharacter(name) == true and HTCommon:IsRetiredRankedCharacter(name) == false then
      table.insert(characterList, name)
    end
  end

  table.sort(characterList)

  if #characterList > 0 then
    HTConsole:SendChatMessage(L["ht-list-all-active-member-mains-title"], "OFFICER", nil, nil)
    HTConsole:SendWrappedMessage(HTData:BuildStringFromTable(characterList, "", ", "), "OFFICER", nil, nil)
  end

end

-- ============================================================================
-- == ListAllRetiredMembers()                                                ==
-- ==                                                                        ==
-- == This function lists all retired member-ranked characters within the    ==
-- == guild.                                                                 ==
-- ============================================================================

function HalonaTools:ListAllRetiredMembers()

  local totalNumberOfGuildMembers = GetNumGuildMembers(true)
  local characterList             = {}

  for index=1, totalNumberOfGuildMembers do
    local name, rank, rankIndex, level, class, zone, note, officernote, online, status = GetGuildRosterInfo(index)
    if HTCommon:IsRetiredRankedCharacter(name) == true then
      table.insert(characterList, name)
    end
  end

  table.sort(characterList)

  if #characterList > 0 then
    HTConsole:SendChatMessage(L["ht-list-all-retired-member-mains-title"], "OFFICER", nil, nil)
    HTConsole:SendWrappedMessage(HTData:BuildStringFromTable(characterList, "", ", "), "OFFICER", nil, nil)
  end

end

-- ============================================================================
-- == ListAllTrials()                                                        ==
-- ==                                                                        ==
-- == This function lists all trial-ranked characters within the             ==
-- == guild.                                                                 ==
-- ============================================================================

function HalonaTools:ListAllTrials()

  local totalNumberOfGuildMembers = GetNumGuildMembers(true)
  local characterList             = {}

  for index=1, totalNumberOfGuildMembers do
    local name, rank, rankIndex, level, class, zone, note, officernote, online, status = GetGuildRosterInfo(index)
    if HTCommon:IsTrialRankedCharacter(name) == true then
      table.insert(characterList, name)
    end
  end

  table.sort(characterList)

  if #characterList > 0 then
    HTConsole:SendChatMessage(L["ht-list-all-trial-mains-title"], "OFFICER", nil, nil)
    HTConsole:SendWrappedMessage(HTData:BuildStringFromTable(characterList, "", ", "), "OFFICER", nil, nil)
  end

end

-- ============================================================================
-- == ListAssociates(characterName)                                          ==
-- ==                                                                        ==
-- == This function examines the supplied characterName parameter to         ==
-- == determine whether the name represents a main-ranked character or an    ==
-- == alt-ranked character.                                                  ==
-- ==                                                                        ==
-- == Where the name represents a main-ranked character, it is used directly ==
-- == as the input parameter to the HalonaTools:ListAlts(characterName)      ==
-- == function.                                                              ==
-- ==                                                                        ==
-- == Where the name represents an alt-ranked character, it is used directly ==
-- == as the input parameter to the utility function                         ==
-- == HTCommon:GetMainCharacterFromAltCharacter(characterName) and the       ==
-- == output from this function is then used directly as the input to the    ==
-- == HalonaTools:ListAlts(characterName) function.                          ==
-- ============================================================================

function HalonaTools:ListAssociates(characterName)

  -- First Check: Is the character in the guild?
  if HTCommon:IsCharacterInGuild(characterName) == false then
    HTConsole:SendChatMessage(string.gsub(L["ht-list-all-alts-not-in-guild"], "CHARACTER_NAME", characterName), "OFFICER", nil, nil)
    return
  end

  -- Second Check: Is the character a main-ranked character?
  if HTCommon:IsMainRankedCharacter(characterName) == true then
    HalonaTools:ListAlts(characterName)
    return
  end

  -- Third Check: Is the character an alt-ranked character?
  if HTCommon:IsAltRankedCharacter(characterName) == true then
    local mainName = HTCommon:GetMainCharacterFromAltCharacter(characterName)
    local messageText = string.gsub(L["ht-list-all-alts-pre-title"], "ALT_NAME", characterName)
          messageText = string.gsub(messageText, "CHARACTER_NAME", mainName)
    HTConsole:SendChatMessage(messageText, "OFFICER", nil, nil)
    HalonaTools:ListAlts(mainName)
    return
  end

  -- Finally: If we got here, something went wrong.
  HTConsole:SendChatMessage(string.gsub(L["ht-list-all-alts-not-searchable"], "CHARACTER_NAME", characterName), "OFFICER", nil, nil)

end

-- ============================================================================
-- == ListAlts(characterName)                                                ==
-- ==                                                                        ==
-- == This function lists all the alt-characters associated with the         ==
-- == supplied main character name.                                          ==
-- ============================================================================

function HalonaTools:ListAlts(characterName)

  -- First Check: Is the character in the guild?
  if HTCommon:IsCharacterInGuild(characterName) == false then
    HTConsole:SendChatMessage(string.gsub(L["ht-list-all-alts-not-in-guild"], "CHARACTER_NAME", characterName), "OFFICER", nil, nil)
    return
  end

  local totalNumberOfGuildMembers = GetNumGuildMembers(true)
  local characterList             = {}

  for index=1, totalNumberOfGuildMembers do
    local name, rank, rankIndex, level, class, zone, note, officernote, online, status = GetGuildRosterInfo(index)
    if HTCommon:IsAltRankedCharacter(name) == true then
      if note == characterName then
        table.insert(characterList, name)
      end
    end
  end

  table.sort(characterList)

  if #characterList > 0 then
    HTConsole:SendChatMessage(string.gsub(L["ht-list-all-alts-title"], "CHARACTER_NAME", characterName), "OFFICER", nil, nil)
    HTConsole:SendWrappedMessage(HTData:BuildStringFromTable(characterList, "", ", "), "OFFICER", nil, nil)
  end

end
